# Copyright (c) Hilscher GmbH. All Rights Reserved.
#
# $Author: bgeorgiev $
# $Date: 2019-05-16 11:34:02 +0300 (Thu, 16 May 2019) $
# $Revision: 654 $


"""
This module provides the connection between the client application and the
remote i/o implementation. It separates use from implementation and might be
useful when implementing user interfaces.

*** Example *** :

```
import time
import htpio.bridge as bridge
import htpio.telnetio as telnetio
import htpio.socketio as socketio

device = telnetio.RaspberryPi('192.168.199.31')
gpio = bridge.RemoteAccess(device)

gpio.lock(14)
gpio.configure(14, telnetio.RaspberryPi.OUTPUT)

gpio.set(14, 1)
time.sleep(1)
gpio.set(14, 0)

gpio.unlock(14)
gpio.deconfigure(14)
gpio.logout()

device = socketio.RaspberryPi('192.168.199.31')
gpio = bridge.RemoteAccess(device)

gpio.lock(15)
gpio.configure(15, socketio.RaspberryPi.OUTPUT)

gpio.set(15, 1)
time.sleep(1)
gpio.set(15, 0)

gpio.unlock(15)
gpio.deconfigure(15)
gpio.logout()

```

"""

# Compatibility imports
from __future__ import unicode_literals
from __future__ import absolute_import

# System imports
from abc import ABCMeta, abstractmethod


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#  CLASS ProtocolInterface : Implementor interface abstraction
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


class Device(object):
    """
    Define the implementor's interface from the bridge pattern. This interface
    provides homogeneous interface for all remote I/O libraries.

    Typically the implementor interface provides only primitive operations,
    and abstraction defines higher-level operations based on these primitives.
    """

    __metaclass__ = ABCMeta

    INPUT = 0
    """ Pin input mode"""

    OUTPUT = 1
    """ Pin output mode"""

    @abstractmethod
    def islocked(self, pin):
        """ Abstract method """
        raise NotImplementedError

    @abstractmethod
    def lock(self, pin):
        """ Abstract method """
        raise NotImplementedError

    @abstractmethod
    def unlock(self, pin):
        """ Abstract method """
        raise NotImplementedError

    @abstractmethod
    def isconfigured(self, pin):
        """ Abstract method """
        raise NotImplementedError

    @abstractmethod
    def configure(self, pin, direction):
        """ Abstract method """
        raise NotImplementedError

    @abstractmethod
    def deconfigure(self, pin):
        """ Abstract method """
        raise NotImplementedError

    @abstractmethod
    def login(self):
        """ Abstract method """
        raise NotImplementedError

    @abstractmethod
    def logout(self):
        """ Abstract method """
        raise NotImplementedError

    @abstractmethod
    def get(self, pin):
        """ Abstract method """
        raise NotImplementedError

    @abstractmethod
    def set(self, pin, value):
        """ Abstract method """
        raise NotImplementedError

    @abstractmethod
    def reset(self, pin):
        """ Abstract method """
        raise NotImplementedError


"""
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#  CLASS Client : Bridge interface for client applications
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
"""


class RemoteAccess(object):

    """
    Define the abstraction's interface from the bridge pattern used by the
    client application.

    Typically the implementor interface provides only primitive operations,
    and abstraction defines higher-level operations based on these primitives.
    """

    def __init__(self, protocol_api):

        # Define instance attributes. These cannot be defined as class
        # attributes as bridge might have different types of in/out
        # and max_gpio definitions.

        self.INPUT = protocol_api.INPUT
        self.OUTPUT = protocol_api.OUTPUT
        self.MAX_GPIO = protocol_api.MAX_GPIO

        # Define interface for remote access
        self._protocol_api = protocol_api

    def islocked(self, pin):
        """
        Check if pin is locked by another process

        *** Example: ***
        ```python
        from htpio.bridge import RemoteAccess
        from htpio import telnetio, socketio

        remoteio = socketio.RaspberryPi('192.168.199.31')
        led = RemoteAccess(remoteio)
        led.lock(14)
        if led.islocked(14):
            print("LED is locked!!!")
        led.logout()
        ...
        ```
        """

        return self._protocol_api.islocked(pin)

    def lock(self, pin):
        """
        Lock pin if it is not used

        *** Example: ***
        ```python
        from htpio.bridge import RemoteAccess
        from htpio import telnetio, socketio

        remoteio = socketio.RaspberryPi('192.168.199.31')
        led = RemoteAccess(remoteio)
        led.lock(14)
        ...
        ```

        """

        self._protocol_api.lock(pin)

    def unlock(self, pin):
        """
        Unlock pin if it belongs to my session

        ***Example***:
        ```python

        from htpio.bridge import RemoteAccess
        from htpio import telnetio, socketio

        remoteio = socketio.RaspberryPi('192.168.199.31')
        led = RemoteAccess(remoteio)
        led.unlock(14)
        ...
        ```
        """

        self._protocol_api.unlock(pin)

    def isconfigured(self, pin):
        """
        Check if pin is configured

        ***Example***:
        ```python

        from htpio.bridge import RemoteAccess
        from htpio import telnetio, socketio

        remoteio = socketio.RaspberryPi('192.168.199.31')
        led = RemoteAccess(remoteio)
        led.isconfigured(14)
        ...
        ```

        """

        return self._protocol_api.isconfigured(pin)

    def configure(self, pin, direction):
        """
        Configure pin with direction if it is free

        ***Example***:
        ```python

        from htpio.bridge import RemoteAccess
        from htpio import telnetio, socketio

        remoteio = socketio.RaspberryPi('192.168.199.31')
        led = RemoteAccess(remoteio)
        led.configure(14, led.OUTPUT)
        led.set(14, 1)
        ...
        ```
        """

        self._protocol_api.configure(pin, direction)

    def deconfigure(self, pin):
        """
        Deconfigure pin if it belongs to my session

        ***Example***:
        ```python

        from htpio.bridge import RemoteAccess
        from htpio import telnetio, socketio

        remoteio = socketio.RaspberryPi('192.168.199.31')
        led = RemoteAccess(remoteio)
        led.deconfigure(14)
        ...
        ```

        """

        self._protocol_api.deconfigure(pin)

    def login(self):
        """
        Registers to remote device

        ***Example***:
        ```python

        from htpio.bridge import RemoteAccess
        from htpio import telnetio, socketio

        remoteio = socketio.RaspberryPi('192.168.199.31')
        led = RemoteAccess(remoteio)
        led.login()
        ...
        ```

        """

        self._protocol_api.login()

    def logout(self):
        """
        Unregisters from remote device

        ***Example***:
        ```python

        from htpio.bridge import RemoteAccess
        from htpio import telnetio, socketio

        remoteio = socketio.RaspberryPi('192.168.199.31')
        led = RemoteAccess(remoteio)
        led.logout()
        ...
        ```
        """

        self._protocol_api.logout()

    def get(self, pin):
        """
        Get pin value

        ***Example***:
        ```python

        from htpio.bridge import RemoteAccess
        from htpio import telnetio, socketio

        remoteio = socketio.RaspberryPi('192.168.199.31')
        led = RemoteAccess(remoteio)
        led.configure(14, led.OUTPUT)
        led.set(14, 1)
        print(led.get(14))
        ...
        ```

        """

        return self._protocol_api.get(pin)

    def set(self, pin, value):
        """
        Write value to pin

        ***Example***:
        ```python

        from htpio.bridge import RemoteAccess
        from htpio import telnetio, socketio

        remoteio = socketio.RaspberryPi('192.168.199.31')
        led = RemoteAccess(remoteio)
        led.configure(14, led.OUTPUT)
        led.set(14, 1)
        print(led.get(14))
        ...
        ```

        """

        self._protocol_api.set(pin, value)

    def reset(self, pin):
        """
        Remove pin lock forcefully

        ***Example***:
        ```python

        from htpio.bridge import RemoteAccess
        from htpio import telnetio, socketio

        remoteio = socketio.RaspberryPi('192.168.199.31')
        led = RemoteAccess(remoteio)
        led.lock(14)
        print(led.islocked(14))
        led.reset(14)
        print(led.islocked(14))
        ...
        ```

        """

        self._protocol_api.reset(pin)




#!/usr/bin/env python

# Copyright (c) Hilscher GmbH. All Rights Reserved.
#
# $Author: bgeorgiev $
# $Date: 2019-05-14 12:32:23 +0300 (Tue, 14 May 2019) $
# $Revision: 592 $


"""
This module implements a command line parser for the "Hardware Test Platform"
from Hilscher.

*Set value using telnet:*

```
htptools -p telnet set --host 192.168.199.31 --pin 14 --val 1
```

*Get value using socket:*
```
python -m htpio -p socket get --host 192.168.199.31 --pin 14 or
htpio -p socket get --host 192.168.199.31 --pin 14
```

*Lock/Unlock pin:*
```
htptools -p telnet lock --host 192.168.199.31 --pin 14
htptools -p socket unlock --host 192.168.199.31 --pin 14
```

"""

# Compatibility imports
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import

# System imports
import argparse

# Local imports
from htpio import bridge, telnetio, socketio


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# PARSER FUNCTIONS
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

# Function get
def func_get(args):

    if args.protocol == 'telnet':
        device = telnetio.RaspberryPi(args.host)

    elif args.protocol == 'socket':
        device = socketio.RaspberryPi(args.host)

    else:
        device = telnetio.RaspberryPi(args.host)

    gpio = bridge.RemoteAccess(device)
    print(gpio.get(args.pin))

# Function set
def func_set(args):

    if args.protocol == 'telnet':
        device = telnetio.RaspberryPi(args.host)

    elif args.protocol == 'socket':
        device = socketio.RaspberryPi(args.host)

    else:
        device = telnetio.RaspberryPi(args.host)

    gpio = bridge.RemoteAccess(device)
    gpio.configure(args.pin, gpio.OUTPUT)
    gpio.set(args.pin, args.val)

# Function lock
def func_lock(args):

    if args.protocol == 'telnet':
        device = telnetio.RaspberryPi(args.host)

    elif args.protocol == 'socket':
        device = socketio.RaspberryPi(args.host)

    else:
        device = telnetio.RaspberryPi(args.host)

    gpio = bridge.RemoteAccess(device)
    gpio.lock(args.pin)

# Function unlock
def func_unlock(args):

    if args.protocol == 'telnet':
        device = telnetio.RaspberryPi(args.host)

    elif args.protocol == 'socket':
        device = socketio.RaspberryPi(args.host)

    else:
        device = telnetio.RaspberryPi(args.host)

    gpio = bridge.RemoteAccess(device)
    gpio.reset(args.pin)

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# PARSER DEFINITION
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

def parser():

    parser = argparse.ArgumentParser(
        description="HTP Library CLI",
        usage="htptools [OPTIONS] COMMAND [ARGS]",
        add_help=False,
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        epilog="For help type htptools [COMMAND] -h | --help"
    )

    group = parser.add_argument_group(title = "OPTIONS")

    # Protocol option
    group.add_argument(
        '-p',
        dest='protocol',
        default='telnet',
        help='Protocol to use (telnet, socket)'
    )

    #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # Define subcommands
    #++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    subparsers = parser.add_subparsers(
        title="COMMAND",
    )

    # Set
    setcmd = subparsers.add_parser('set', help='set pin value')
    setcmd.add_argument('--host', default='192.168.199.31', help="server address")
    setcmd.add_argument('--port', default=23, help="server port")
    setcmd.add_argument('--pin', type=int, help='pin number')
    setcmd.add_argument('--val', type=int, help='signal value')
    setcmd.set_defaults(func=func_set)

    # Get
    getcmd = subparsers.add_parser('get', help='get pin value')
    getcmd.add_argument('--host', default='192.168.199.31', help="server address")
    getcmd.add_argument('--port', default=23, help="server port")
    getcmd.add_argument('--pin', type=int, help='pin number')
    getcmd.set_defaults(func=func_get)

    # Lock
    lockcmd = subparsers.add_parser('lock', help='locks pin')
    lockcmd.add_argument('--host', default='192.168.199.31', help="server address")
    lockcmd.add_argument('--port', default=23, help="server port")
    lockcmd.add_argument('--pin', type=int, help='pin number')
    lockcmd.set_defaults(func=func_lock)

    # Unlock
    unlockcmd = subparsers.add_parser('unlock', help='unlocks pin')
    unlockcmd.add_argument('--host', default='192.168.199.31', help="server address")
    unlockcmd.add_argument('--port', default=23, help="server port")
    unlockcmd.add_argument('--pin', type=int, help='pin number')
    unlockcmd.set_defaults(func=func_unlock)

    # cmd_01 = "-p socket set --host 192.168.199.31 --port 23 --pin 14 --val 1"
    # args = parser.parse_args(cmd_01.split())

    # cmd_02 = "-p socket set --host 192.168.199.31 --port 23 --pin 14 --val 0"
    # args = parser.parse_args(cmd_02.split())

    # cmd_03 = "-p socket lock --host 192.168.199.31 --port 23 --pin 14"
    # args = parser.parse_args(cmd_03.split())

    # cmd_04 = "-p socket unlock --host 192.168.199.31 --port 23 --pin 14"
    # args = parser.parse_args(cmd_04.split())

    args = parser.parse_args()

    try:
        args.func(args)

    except AttributeError:
        parser.print_help()


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# MAIN PROGRAM
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

main = parser

if __name__ == '__main__':
    main()
